import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from '../../../../state/reducers'
import SimpleCodeEditorWrapper from '../../../codeEditors/simpleCodeEditorWrapper'
import {tutorFeedbackEditorDebounceInMs, tutorViewTutorFeedbackForStudentEditor} from '../../../../constants'
import {setFeedbackForStudent} from '../../../../state/actions/tutorViewSite/sub/setAssessmentActions'
import {EditorTab} from '../../../codeEditors/multiTabCodeEditorProps'
import {KnownInternalTestTypes} from '../../../../types/testTypes'
import {
  aceCompareFilesTestLanguage,
  aceRegexLanguage,
  aceTestProtocolLanguage,
  aceTextLanguage
} from '../../../../helpers/editorHelper'
import {AceEditorsStorageHelper} from '../../../codeEditors/aceEditors/aceEditorsStorageHelper'

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    releaseFinishedAndAutomaticAssessmentHasFinished: !rootState.tutorViewSiteState.exercise.exerciseRelease.isReleased &&
      rootState.tutorViewSiteState.exercise.exerciseRelease.hasAutomaticAssessmentFinished,
    assessment: rootState.tutorViewSiteState.assessment,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setFeedbackForStudent,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class FeedbackEditorView extends React.Component<Props, any> {
  render(): JSX.Element {

    //when we load (also when we click next/previous) submission we clear all AceEditorsStorageHelper
    //when we load the tutor view with the initial submission we init the feedback editor with the initial feedback (empty)
    //and we switch to that empty feedback (ace session)

    const tempEditorTab: EditorTab = {
      id: 0, //this.props.assessment.userId
      content: (this.props.assessment.hasAssessment && this.props.assessment.feedbackForStudent !== null)
               ? this.props.assessment.feedbackForStudent
               : '',
      language: 'markdown',
      isReadonly: this.props.releaseFinishedAndAutomaticAssessmentHasFinished === false || this.props.assessment.hasAssessment === false,
      fileNameWithExtensions: '',
      displayIndex: 1,
      isMainFile: true,
      isContentVisibleForUser: true,
    }

    const editorTabs = AceEditorsStorageHelper.getEditor(tutorViewTutorFeedbackForStudentEditor)

    if (editorTabs && editorTabs.length === 1 && editorTabs[0].tab.id === -1) {
      //this was set by the initial tutor view load with the default assessment
      //because we mounted the feedback editor and then changed tabs...
      AceEditorsStorageHelper.removeEditor(tutorViewTutorFeedbackForStudentEditor)
      AceEditorsStorageHelper.addEditor(tutorViewTutorFeedbackForStudentEditor, tempEditorTab)

    } else if (editorTabs === undefined) { //when we load another submission but the feedback editor is visible --> no mount --> no default session
      //so create a default session (simple editor will re-use the this session if the associated tab has the same id)
      //this allows us to clear all e.g. undo else we would undo and get the previous/next submission feedback
      AceEditorsStorageHelper.addEditor(tutorViewTutorFeedbackForStudentEditor, tempEditorTab)
    }

    return (
      <div className={`fh${this.props.assessment.hasAssessment ? '' : ' div-disabled'}`} style={{overflow: 'hidden'}}>

        <SimpleCodeEditorWrapper
          id={tutorViewTutorFeedbackForStudentEditor}
          editorTab={tempEditorTab}
          onBlur={(content: string, editorTabId: number) => {
            this.props.setFeedbackForStudent(content)
          }}
          debounceTimeInMs={tutorFeedbackEditorDebounceInMs}

        />

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(FeedbackEditorView)
